/*
 * ========================================================================
 * (c) Copyright 2021 Micro Focus or one of its affiliates.
 * ========================================================================
 */
//Most of the functions and structure were taken from Review Details' jsonHandler.js file
  var getAttributeText = function(attr) { //This was edited to use Identity attributes.
    if(attr !== null) {
          if(attr.indexOf('ext_') !== -1) {
              return attr.substring(4);
          }
          
        switch (attr) {
            case 'affiliatedId':
              return 'AFFILIATED_ID';
            case 'city':
                return 'CITY';
            case 'country':
                return 'COUNTRY';
            case 'createDate':
                return 'CREATE_DATE';
            case 'deleteDate':
                return 'DELETE_DATE';
            case 'department':
                return 'DEPARTMENT';
            case 'displayName':
                return 'DISPLAY_NAME';
            case 'dn':
                return 'DN';
            case 'originalCreateDate':
                return 'ORIGINAL_CREATE_DATE';
            case 'state':
                return 'STATE';
            case 'supervisor':
                return 'SUPERVISOR';
            case 'jobTitle':
                return 'JOB_TITLE';
            case 'workforceId':
                return 'WORKFORCE_ID';
            case 'risk':
                return 'RISK';
            default:
                return attr;
        }
        
    }                            
  };
  
  var getOperatorText = function(op) {
    if (op != null) {
        switch (op) {
            case 'GT':
                return 'GREATER_THAN';
            case 'LT':
                return 'LESS_THAN';
            case 'EQ':
                return 'EQUAL_TO';
            case 'GE':
                return 'GREATER_THAN_EQ';
            case 'LE':
                return 'LESS_THAN_EQ';
            case 'MATCHES':
                return 'MATCHES';
            case 'NE':
                return 'NOT_EQUAL';
            case 'ADD':
                return "+";
            case 'SUB':
                return "-";
            case 'MULT':
                return '*';
            case 'DIV':
                return '/';
            case 'MOD':
                return '%';
            default:
                return op;
        }
    }                        
  };

  var parseEpoch = function (str)
  {
    var d = new Date(str * 1000);
    return d;
  }
  
  
  var evaluateExpression = function(expr){ //WIP
    resultList = [];
    
    var operands1 = expr.value.operands;
    var operator = expr.value.operator;
    var count = 1;
    if (expr.value.operands !== undefined){
        operands1.forEach( function(operands){
                if (operands.type == "EXPRESSION"){
                    resultList.push("(");
                    resultList = resultList.concat(evaluateExpression(operands));
                    //We use the concat function here to add a list onto a list
                    //.push in this case kind of excludes some info.
                    resultList.push(")");
                }//Use recursion to put in another expression in parenthesis.
                 //This won't exactly mimic what's in IG, but it should return the same order of operations.
                 
                else if (operands.type == "FUNCTION"){
                    resultList.push(operands.value.funcType);
                } //Push the function type with the operator from earlier
                  
                else if (operands.type == "DECIMAL_NUMBER"){
                    resultList.push(operands.value);
                } //Just push the value of the number
                

                //In between iterations, push the operator.
                 //If it's the last value of operands, don't push it.

                if (count !== operands1.length){
                    resultList.push(getOperatorText(operator));
                }
                count += 1;
                
            }
        )
    }
    
    return resultList;
  }
  
  var parseInfo = function (jsonObj) {
    var resultList = [];
    

    if(jsonObj.childCriteria !== undefined) {
        
        if (jsonObj.operator == "NOT"){   //To mimic the display on IG i.e not (x = y or a = b)
            resultList.push("not (");
        }

        var children = jsonObj.childCriteria;
        var child_length = children.length;
        var count_children = 1;

        children.forEach(
            function(children) {
                if(children.attributeKey !== undefined) {
                
                    if(children.attributeKey == "createDate" || children.attributeKey == "originalCreateDate" || children.attributeKey == "deleteDate") {
                        resultList.push(getAttributeText(children.attributeKey));

                        if(children.operator !== undefined) {
                            resultList.push(getOperatorText(children.operator));
                        }
                        
                        if(children.value !== undefined) {
                            resultList.push(parseEpoch(children.value));
                        }
                        else if (children.expr !== undefined){
                            resultList = resultList.concat(evaluateExpression(children.expr));
                            //We use the concat function here to add a list onto a list
                            //.push won't work well here. It makes it ugly and kind of unreadable.
                        }
                    }
                    else if(children.value !== undefined) {
                        resultList.push(getAttributeText(children.attributeKey));

                        if(children.operator !== undefined) {
                            resultList.push(getOperatorText(children.operator));
                        }
                        if(children.value !== undefined) {
                            resultList.push(children.value);
                        }
                    }
                    //This takes care of the supervisor case for attribute key since instead of 'value' 'values'
                    //it has 'entities'
                    else if(children.attributeKey == "supervisor"){
                        resultList.push(getAttributeText(children.attributeKey));
                        
                        if(children.operator !== undefined) {
                           resultList.push(getOperatorText(children.operator));
                        }
                        if (children.valueAttrKey !== undefined){
                            resultList.push(getAttributeText(children.valueAttrKey));
                        }
                        else if(children.entities !== undefined) {
                            var data_values = children.entities;
                            var count = 1;
                            
                            data_values.forEach(
                                function(data_values) {
                                    
                                    if(count != children.entities.length) {
                                        if (count == 1) {
                                            resultList.push('[');
                                        }
                                        resultList.push("'",data_values.displayName,"'",  jsonObj.operator);
                                        count += 1;
                                    }
                                    //
                                    // case of single value
                                    else if ((count == children.entities.length) && (children.entities.length == 1)) {
                                        resultList.push("'",data_values.displayName, "'");
                                    }
                                    // case of reaching the last element in a multiple values list
                                    else {
                                        resultList.push( "'", data_values.displayName, "'", ']');
                                    }
                                }
                            )
                        }
                    }
                    
                    //takes care of "values" json attribute (that is neither value or have attribute key as 
                    //createDate, originalDate nor deleteDate)
                    else {  
                        resultList.push(getAttributeText(children.attributeKey));
                        
                        if(children.operator !== undefined) {
                             resultList.push(getOperatorText(children.operator));
                        }
                        if (children.valueAttrKey !== undefined){
                            resultList.push(getAttributeText(children.valueAttrKey));
                        }
                        else if(children.values !== undefined) {
                            var data_values = children.values;
                            var count = 1;
                            
                            data_values.forEach(
                                function(data_values) {
                                    
                                    if(count != children.values.length) {
                                        if (count == 1) {
                                            resultList.push('[');
                                        }
                                        resultList.push("'",data_values,"'",  jsonObj.operator);
                                        count += 1;
                                    }
                                    //
                                    // case of single value
                                    else if ((count == children.values.length) && (children.values.length == 1)) {
                                        resultList.push("'",data_values, "'");
                                    }
                                    // case of reaching the last element in a multiple values list
                                    else {
                                        resultList.push( "'", data_values, "'", ']');
                                    }
                                }
                            )
                        }
                    }
                }//This recursively calls the same function if there is a nested filter.
                else if (children.childCriteria !== undefined){
                        resultList.push("(");
                        resultList = resultList.concat(parseInfo(children));
                        resultList.push(")");
                    }
                //joining each criteria statement with the selected operator

                //for the case when the current children item is not the last children
                if(count_children != child_length) {
                    jsonObj.operator == "NOT" ? resultList.push("or",jsonObj.operator): resultList.push(jsonObj.operator);
                    count_children += 1;
                }
            
                else if (count_children == child_length && jsonObj.operator == "NOT"){
                    resultList.push(")"); //Adds in that last parenthesis. See comment in localizeJSON
                }
            }
        )
    } 
    return resultList;  
  }

  var localizeJSON = function(jsonStr, localizer) {
    var jsonObj = JSON.parse(jsonStr);
    var resultList = [];
    if(jsonObj !== null && localizer !== null){
        
        
        resultList = resultList.concat(parseInfo(jsonObj));
               
      return resultList.map(function(str) {
         return localizer.getString(str);
      }).join(' ');
    }
  }
  
